"""
 Sample python script for reading NetCDF files in the Gonzalez et al. 2020 dataset
 Sample plotting example
 
Hannah Bloomfield - UoR - Jan 2021


Example loading script for European weather and energy data

Once you run this you will have both hindcast and ERA5 reanalysis variables 
loaded in for matching dates. 
Country names and forecast date are also loaded and converted into a useable 
format.


"""

import numpy as np
from netCDF4 import Dataset
import matplotlib.pyplot as plt

# the directory where the data is stored in - change to match your location!
data_dir = '/YOUR_DATASET_LOCATION/country_daily_demand/'

# the name of the file you wish to load
file_name = 'ecmwf_s2s_coun_demand_fcst_w_VI_and_rea_DAILY_stmon_01.nc'

# load in the .nc file
dataset = Dataset(data_dir + file_name)

# this command gives the short names of the .nc file contents
dataset.variables.keys()
# load the forecast of the demand data shape of array. The array dimensions are: 
# [country, day , year , fcst start , ensemble member ]
# Units = GW
demand_fcst = dataset.variables['demand_fcst'][:]
# load the forecast of the demand data shape of array. The array dimensions are: 
# [country, day , year , fcst start ] 
# Units = GW
demand_rea = dataset.variables['demand_rea'][:]
# load the dates that each of the forecasts are valid for. The array dimensions are: 
# [day , year , fcst start ] 
# Units = days since 01,01,1970, the same as in ERA5.
date = dataset.variables['date'][:]
# load the identifying codes for each country.
country_codes = dataset.variables['country_codes'][:]

dataset.close()


# convert the date handle into something more useful
# set the initial date as the '0' date in the dates.
initial_date = np.datetime64('1970-01-01')
#setup a nump.datetime array to store the new dates in
datetime_array = np.zeros(np.shape(date),dtype='datetime64[D]')

#loop through the dimensions and assign the dates in the array
for day_i in range(0,44):
    for year_i in range(0,20):
        for start_i in range(0,8):

            datetime_array[day_i, year_i, start_i] = initial_date + \
            np.timedelta64(np.int(date[day_i, year_i, start_i]),'D')


# convert the country codes into country names using a dictionary
country_dictionary = {
    country_codes[0][0] + country_codes[0][1] : 'Austria',
    country_codes[1][0] + country_codes[1][1] : 'Belgium',
    country_codes[2][0] + country_codes[2][1] : 'Bulgaria',
    country_codes[3][0] + country_codes[3][1] : 'Croatia',
    country_codes[4][0] + country_codes[4][1] : 'Czech Republic',
    country_codes[5][0] + country_codes[5][1] : 'Denmark',
    country_codes[6][0] + country_codes[6][1] : 'Finland',
    country_codes[7][0] + country_codes[7][1] : 'France',
    country_codes[8][0] + country_codes[8][1] : 'Germany',
    country_codes[9][0] + country_codes[9][1] : 'Greece',
    country_codes[10][0] + country_codes[10][1] : 'Hungary',
    country_codes[11][0] + country_codes[11][1] : 'Ireland',
    country_codes[12][0] + country_codes[12][1] : 'Italy',
    country_codes[13][0] + country_codes[13][1] : 'Latvia',
    country_codes[14][0] + country_codes[14][1] : 'Lithuania',
    country_codes[15][0] + country_codes[15][1] : 'Luxembourg',
    country_codes[16][0] + country_codes[16][1] : 'Montenegro',
    country_codes[17][0] + country_codes[17][1] : 'Netherlands',
    country_codes[18][0] + country_codes[18][1] : 'Norway',
    country_codes[19][0] + country_codes[19][1] : 'Poland',
    country_codes[20][0] + country_codes[20][1] : 'Portugal',
    country_codes[21][0] + country_codes[21][1] : 'Romania',
    country_codes[22][0] + country_codes[22][1] : 'Slovakia',
    country_codes[23][0] + country_codes[23][1] : 'Slovenia',
    country_codes[24][0] + country_codes[24][1] : 'Spain',
    country_codes[25][0] + country_codes[25][1] : 'Sweden',
    country_codes[26][0] + country_codes[26][1] : 'Switzerland',
    country_codes[27][0] + country_codes[27][1] : 'United Kingdom',

}

# create an empty list to assign country names to.
country_full_names = []
# assign country names
for count_i in range(0,len(country_codes)):
    country_full_names.append( country_dictionary[country_codes[count_i][0] + 
                                                  country_codes[count_i][1]])




# make an example plot of the data

fig = plt.figure(figsize=(10,5))
plt.plot(datetime_array[:,0,0],demand_fcst[0,:,0,0,:],color='grey')
# plot just one member for the legend
plt.plot(datetime_array[:,0,0],demand_fcst[0,:,0,0,0],color='grey',
         label='ensemble hindcast')
plt.plot(datetime_array[:,0,0],demand_rea[0,:,0,0],color='k',label='ERA5')
plt.ylabel('Demand (GW)',fontsize=16)
plt.xlabel('Date',fontsize=16)
plt.title(country_full_names[0],fontsize=18)
plt.legend(frameon=False)
plt.show()
